var Pathname, extractCallback, flatten, mods;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice;
mods = {
  fs: require('fs'),
  path: require('path'),
  util: require('util')
};
flatten = function(array) {
  return array.reduce((__bind(function(memo, value) {
    if (Array.isArray(value)) {
      value = flatten(value);
    }
    return memo.concat(value);
  }, this)), []);
};
extractCallback = function() {
  var args, i, k, v, _len;
  args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
  for (k = 0, _len = args.length; k < _len; k++) {
    v = args[k];
    if ((v != null ? v.constructor : void 0) === Function) {
      i = k;
    }
  }
  return [args[i]].concat(__slice.call(args.slice(0, i)));
};
Pathname = (function() {
  var func, _fn, _i, _len, _ref;
  function Pathname(path) {
    this.path = mods.path.normalize(path.toString()).replace(/(.)\/$/, '$1');
  }
  Pathname.prototype.join = function() {
    var paths, _ref;
    paths = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    return new this.constructor((_ref = mods.path).join.apply(_ref, [this.path].concat(__slice.call(paths.map(function(path) {
      return path.toString();
    })))));
  };
  Pathname.prototype.dirname = function() {
    return new this.constructor(mods.path.dirname(this.path));
  };
  Pathname.prototype.basename = function(ext) {
    return new this.constructor(mods.path.basename(this.path, ext));
  };
  Pathname.prototype.extname = function() {
    return mods.path.extname(this.path);
  };
  Pathname.prototype.exists = function(cb) {
    if (cb != null) {
      return mods.path.exists(this.path, cb);
    } else {
      return mods.path.existsSync(this.path);
    }
  };
  Pathname.prototype.stat = function(cb) {
    if (cb != null) {
      return mods.fs.stat(this.path, cb);
    } else {
      return mods.fs.statSync(this.path);
    }
  };
  Pathname.prototype.lstat = function(cb) {
    if (cb != null) {
      return mods.fs.lstat(this.path, cb);
    } else {
      return mods.fs.lstatSync(this.path);
    }
  };
  Pathname.prototype.realpath = function(cb) {
    if (cb != null) {
      return mods.fs.realpath(this.path, __bind(function(err, resolvedPath) {
        if (err != null) {
          return cb(err, null);
        } else {
          return cb(null, new this.constructor(resolvedPath));
        }
      }, this));
    } else {
      return new this.constructor(mods.fs.realpathSync(this.path));
    }
  };
  Pathname.prototype.unlink = function(cb) {
    if (cb != null) {
      return mods.fs.unlink(this.path, __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.unlinkSync(this.path);
      return this;
    }
  };
  Pathname.prototype.rmdir = function(cb) {
    if (cb != null) {
      return mods.fs.rmdir(this.path, __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.rmdirSync(this.path);
      return this;
    }
  };
  Pathname.prototype.mkdir = function(mode, cb) {
    var _ref;
    _ref = extractCallback(mode, cb), cb = _ref[0], mode = _ref[1];
        if (mode != null) {
      mode;
    } else {
      mode = 0700;
    };
    if (cb != null) {
      return mods.fs.mkdir(this.path, mode, __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.mkdirSync(this.path, mode);
      return this;
    }
  };
  Pathname.prototype.open = function(flags, mode, cb) {
    var _ref;
    _ref = extractCallback(flags, mode, cb), cb = _ref[0], flags = _ref[1], mode = _ref[2];
        if (mode != null) {
      mode;
    } else {
      mode = 0666;
    };
    if (cb != null) {
      if (this.fd != null) {
        return process.nextTick(__bind(function() {
          return cb(null, this.fd);
        }, this));
      } else {
        return mods.fs.open(this.path, flags, mode, __bind(function(err, fd) {
          this.fd = fd;
          return cb(err, this.fd);
        }, this));
      }
    } else {
      if (this.fd != null) {
        return this.fd;
      }
      return this.fd = mods.fs.openSync(this.path, flags, mode);
    }
  };
  Pathname.prototype.close = function(cb) {
    if (cb != null) {
      if (this.fd != null) {
        mods.fs.close(this.fd, __bind(function(err) {
          return cb(err, this);
        }, this));
        return delete this.fd;
      } else {
        return cb(null, this);
      }
    } else {
      if (this.fd != null) {
        mods.fs.closeSync(this.fd);
        delete this.fd;
        return this;
      }
    }
  };
  Pathname.prototype.rename = function(path, cb) {
    var _ref;
    _ref = extractCallback(path, cb), cb = _ref[0], path = _ref[1];
    if (cb != null) {
      return mods.fs.rename(this.path, path.toString(), __bind(function(err) {
        return cb(err, new this.constructor(path.toString()));
      }, this));
    } else {
      mods.fs.renameSync(this.path, path.toString());
      return new this.constructor(path.toString());
    }
  };
  Pathname.prototype.truncate = function(len, cb) {
    var _ref;
    _ref = extractCallback(len, cb), cb = _ref[0], len = _ref[1];
    if (cb != null) {
      return this.open('r+', 0666, __bind(function(err, fd) {
        if (err != null) {
          return cb(err);
        } else {
          return mods.fs.truncate(fd, len, __bind(function(err) {
            return cb(err, this);
          }, this));
        }
      }, this));
    } else {
      mods.fs.truncateSync(this.open('r+', 0666), len);
      return this;
    }
  };
  Pathname.prototype.chmod = function(mode, cb) {
    var _ref;
    _ref = extractCallback(mode, cb), cb = _ref[0], mode = _ref[1];
    if (cb != null) {
      return mods.fs.chmod(this.path, mode, __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.chmodSync(this.path, mode);
      return this;
    }
  };
  Pathname.prototype.readFile = function(encoding, cb) {
    var _ref;
    _ref = extractCallback(encoding, cb), cb = _ref[0], encoding = _ref[1];
    if (cb != null) {
      return mods.fs.readFile(this.path, encoding, cb);
    } else {
      return mods.fs.readFileSync(this.path, encoding);
    }
  };
  Pathname.prototype.writeFile = function(data, encoding, cb) {
    var _ref;
    _ref = extractCallback(data, encoding, cb), cb = _ref[0], data = _ref[1], encoding = _ref[2];
    if (cb != null) {
      return mods.fs.writeFile(this.path, data, encoding, __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.writeFileSync(this.path, data, encoding);
      return this;
    }
  };
  Pathname.prototype.link = function(dstpath, cb) {
    var _ref;
    _ref = extractCallback(dstpath, cb), cb = _ref[0], dstpath = _ref[1];
    if (cb != null) {
      return mods.fs.link(this.path, dstpath.toString(), __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.linkSync(this.path, dstpath.toString());
      return this;
    }
  };
  Pathname.prototype.symlink = function(dstpath, cb) {
    var _ref;
    _ref = extractCallback(dstpath, cb), cb = _ref[0], dstpath = _ref[1];
    if (cb != null) {
      return mods.fs.symlink(this.path, dstpath.toString(), __bind(function(err) {
        return cb(err, this);
      }, this));
    } else {
      mods.fs.symlinkSync(this.path, dstpath.toString());
      return this;
    }
  };
  Pathname.prototype.readlink = function(cb) {
    if (cb != null) {
      return mods.fs.readlink(this.path, __bind(function(err, path) {
        return cb(err, new this.constructor(path));
      }, this));
    } else {
      return new this.constructor(mods.fs.readlinkSync(this.path));
    }
  };
  Pathname.prototype.readdir = function(cb) {
    if (cb != null) {
      return mods.fs.readdir(this.path, __bind(function(err, paths) {
        if (err != null) {
          return cb(err, null);
        } else {
          return cb(null, paths.map(__bind(function(path) {
            return new this.constructor(path);
          }, this)));
        }
      }, this));
    } else {
      return mods.fs.readdirSync(this.path).map(__bind(function(path) {
        return new this.constructor(path);
      }, this));
    }
  };
  Pathname.prototype.watchFile = function() {
    var args, _ref;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    (_ref = mods.fs).watchFile.apply(_ref, [this.path].concat(__slice.call(args)));
    return this;
  };
  Pathname.prototype.unwatchFile = function() {
    mods.fs.unwatchFile(this.path);
    return this;
  };
  _ref = ['isFile', 'isDirectory', 'isBlockDevice', 'isCharacterDevice', 'isFIFO', 'isSocket'];
  _fn = __bind(function(func) {
    return this.prototype[func] = function(cb) {
      if (cb != null) {
        return this.exists(__bind(function(exists) {
          if (exists) {
            return this.stat(function(err, stats) {
              return cb(err, stats[func]());
            });
          } else {
            return cb(null, false);
          }
        }, this));
      } else {
        return this.exists() && this.stat()[func]();
      }
    };
  }, Pathname);
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    func = _ref[_i];
    _fn(func);
  }
  Pathname.prototype.isSymbolicLink = function(cb) {
    if (cb != null) {
      return this.lstat(function(err, stats) {
        if (/^ENOENT/.test(err != null ? err.message : void 0)) {
          return cb(null, false);
        } else {
          return cb(err, stats != null ? stats.isSymbolicLink() : void 0);
        }
      });
    } else {
      try {
        return this.lstat().isSymbolicLink();
      } catch (e) {
        return false;
      }
    }
  };
  Pathname.prototype.toString = function() {
    return this.path;
  };
  Pathname.prototype.parent = function() {
    return this.dirname();
  };
  Pathname.prototype.touch = function(mode, cb) {
    var _ref2;
    _ref2 = extractCallback(mode, cb), cb = _ref2[0], mode = _ref2[1];
    if (cb != null) {
      return this.open('w+', mode, __bind(function(err, _) {
        return cb(err, this);
      }, this));
    } else {
      this.open('w+', mode);
      return this.close();
    }
  };
  Pathname.prototype.tree = function(depth, cb) {
    var done, paths, _ref2;
    _ref2 = extractCallback(depth, cb), cb = _ref2[0], depth = _ref2[1];
    done = false;
    paths = [this];
    if (cb != null) {
      if (!this.isSymbolicLink() && this.isDirectory() && (!(depth != null) || depth > 0)) {
        return mods.fs.readdir(this.path, __bind(function(err, files) {
          var count;
          if (done) {
            return;
          }
          if (err != null) {
            done = true;
            return cb(err, null);
          } else {
            count = files.length;
            return files.forEach(__bind(function(fname) {
              return this.join(fname).tree(depth && (depth - 1), function(err, _paths) {
                if (done) {
                  return;
                }
                if (err != null) {
                  done = true;
                  return cb(err, null);
                } else {
                  paths.push(_paths);
                  if (--count === 0) {
                    return cb(null, flatten(paths));
                  }
                }
              });
            }, this));
          }
        }, this));
      } else {
        return cb(null, paths);
      }
    } else {
      if (!this.isSymbolicLink() && this.isDirectory() && (!(depth != null) || depth > 0)) {
        mods.fs.readdirSync(this.path).forEach(__bind(function(fname) {
          return paths.push(this.join(fname).tree(depth && (depth - 1)));
        }, this));
      }
      return flatten(paths);
    }
  };
  Pathname.prototype.rmR = function(cb) {
    if (cb != null) {
      return this.tree(__bind(function(err, files) {
        if (err != null) {
          return cb(err, null);
        } else {
          files.reverse().forEach(function(path) {
            if (path.isSymbolicLink()) {
              path.unlink();
            }
            if (path.isFile()) {
              path.unlink();
            }
            if (path.isDirectory()) {
              return path.rmdir();
            }
          });
          return cb(null, this);
        }
      }, this));
    } else {
      this.tree().reverse().forEach(function(path) {
        if (path.isSymbolicLink()) {
          path.unlink();
        }
        if (path.isFile()) {
          path.unlink();
        }
        if (path.isDirectory()) {
          return path.rmdir();
        }
      });
      return this;
    }
  };
  Pathname.prototype.mkdirP = function(cb) {
    var create;
    create = __bind(function() {
      return this.traverse(function(path) {
        if (!path.exists()) {
          return path.mkdir();
        }
      });
    }, this);
    if (cb != null) {
      return process.nextTick(__bind(function() {
        var p;
        try {
          p = create();
        } catch (e) {
          cb(e, null);
        }
        return cb(null, p);
      }, this));
    } else {
      return create();
    }
  };
  Pathname.prototype.traverse = function(cb) {
    return cb(this.components().map(__bind(function(path) {
      return new this.constructor(path);
    }, this)).reduce(function(curr, next) {
      cb(curr);
      return curr.join(next);
    }));
  };
  Pathname.prototype.components = function() {
    var elements;
    elements = this.path.split('/').filter(function(e) {
      return e.length !== 0;
    });
    if (this.path[0] === '/') {
      elements.unshift('/');
    }
    return elements;
  };
  Pathname.prototype.children = function() {
    var args;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    return this.readdir.apply(this, args);
  };
  Pathname.prototype.siblings = function(cb) {
    if (cb != null) {
      return this.parent().children(__bind(function(err, paths) {
        var _paths;
        if (err != null) {
          return cb(err, null);
        } else {
          _paths = paths.filter(__bind(function(path) {
            return path.toString() !== this.basename().toString();
          }, this));
          return cb(null, _paths);
        }
      }, this));
    } else {
      return this.parent().children().filter(__bind(function(path) {
        return path.toString() !== this.basename().toString();
      }, this));
    }
  };
  return Pathname;
}).call(this);
exports.Pathname = Pathname;